<?php

namespace TDevAgency\Cms\Controllers\Admin;

use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\View;
use TDevAgency\Cms\Controllers\Controller;
use TDevAgency\Cms\Models\BaseModels\Post;
use TDevAgency\Cms\Models\BaseModels\PostLocalization;
use TDevAgency\Cms\Traits\CategoryTrait;
use TDevAgency\Cms\Traits\PostHelper;
use Throwable;

class PageController extends Controller
{
    use CategoryTrait;
    use PostHelper;

    /**
     * @param Request $request
     * @return \Illuminate\Contracts\View\View
     */
    public function index(Request $request): \Illuminate\Contracts\View\View
    {
        return $this->displayList(
            Post::PAGE_TYPE,
            $request,
            ['index' => 'cms.pages.index', 'edit' => 'cms.pages.edit', 'destroy' => 'cms.pages.destroy']
        );
    }

    /**
     * @return \Illuminate\Contracts\View\View
     */
    public function create(): \Illuminate\Contracts\View\View
    {
        return View::make('cms::admin.screens.pages.single')->with('post', new Post());
    }

    /**
     * @param Request $request
     */
    public function store(Request $request): ?RedirectResponse
    {
        $this->validate($request, $this->validatorsArray());

        $page = new Post();
        try {
            DB::transaction(function () use ($request, &$page) {
                $this->postPrepare($request, $page);
                $page->post_type = Post::PAGE_TYPE;
                $page->save();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $page);
                }
            });

            return Redirect::route('cms.pages.index')
                ->with('toast', [
                    'title' => trans('cms::admin.toast.created'),
                    'status' => 'success'
                ]);
        } catch (Throwable $e) {
            Log::error($e);
            return Redirect::back()->withInput()
                ->with('toast', [
                    'title' => trans('cms::admin.toast.not_created'),
                    'status' => 'danger'
                ]);
        }
    }

    /**
     * @param Post $page
     * @return \Illuminate\Contracts\View\View
     */
    public function edit(Post $page): \Illuminate\Contracts\View\View
    {
        $page->setRelation(
            'localizations',
            $page->localizations->mapWithKeys(function (PostLocalization $pageLocalization) {
                return [$pageLocalization->lang => $pageLocalization];
            })
        );
        return View::make('cms::admin.screens.pages.single')
            ->with('post', $page);
    }

    /**
     * @param Request $request
     * @param Post $page
     * @return RedirectResponse
     */
    public function update(Request $request, Post $page): ?RedirectResponse
    {
        $this->validate($request, $this->validatorsArray($page->id));

        try {
            DB::transaction(function () use ($request, &$page) {
                $this->postPrepare($request, $page);
                $page->update();

                if ($request->has('localizations')) {
                    $this->saveLocalization($request->get('localizations'), $page);
                }
            });
            return Redirect::route('cms.pages.index')->with('toast', [
                'title' => trans('cms::admin.toast.updated'),
                'status' => 'success'
            ]);
        } catch (Throwable $e) {
            Log::error($e);
            return Redirect::back()->withInput()->with('toast', [
                'title' => trans('cms::admin.toast.not_updated'),
                'status' => 'danger'
            ]);
        }
    }

    /**
     * @param Post $page
     * @return RedirectResponse
     * @throws Throwable
     */
    public function destroy(Post $page)
    {
        if ($page->slug !== '/') {
            try {
                DB::transaction(function () use ($page) {
                    PostLocalization::where('post_id', $page->id)->delete();
                    Post::where('id', $page->id)->delete();
                });
            } catch (Throwable $exception) {
                return Redirect::back()->with('toast', [
                    'title' => trans('cms::admin.toast.not_deleted'),
                    'status' => 'warning'
                ]);
            }

            return Redirect::back()->with('toast', [
                'title' => trans('cms::admin.toast.deleted'),
                'status' => 'success'
            ]);
        }
        return Redirect::back()->with('toast', [
            'title' => trans('cms::admin.toast.can_not_delete_slash'),
            'status' => 'danger'
        ]);
    }
}
